%
% This example illustrates the use of PPA on a 3D Helix data. 
%
% - First PPA models with different polynomial orders are learnt.
%
%   The models are employed in order to reduce the data dimensionality to one dimension.
%   Results of the data and the dimensionality reduced data are shown in the original
%   and in the PPA domain.
%
% - The same experiment is performed but reducing the data to two dimensions. 
%
% - Finally, an example of how to apply PPA to new data is shown.
%
% Valero Laparra, 2012 (c)
% valero.laparra@uv.es
% http://isp.uv.es
%

clear all
close all

%% DATA: NOISY HELIX 

a = 3;
b = 2;
t = linspace(0,4*pi,2000);
x = a*cos(t);
y = a*sin(t);
z = b*t;

helix = [z; x; y];

dat = helix + 0.4*randn(3,length(helix(1,:)));

figure
plot3(dat(1,:),dat(2,:),dat(3,:),'b.')
axis equal
% hold on, plot3(raspa(1,:),raspa(2,:),raspa(3,:),'r.')

%% PPA 1D
for ord =1:10
    [datT Results] = PPA(dat,ord,3);

    %%
    idat = inv_PPA(datT,Results,1);
    
    figure  
    plot3(dat(1,:),dat(2,:),dat(3,:),'b.')
    axis equal
    hold on
    plot3(idat(1,:),idat(2,:),idat(3,:),'r.')
    title({['PPA, 1D reduction, ord: ' num2str(ord)],'Original data, blue','Data reduced to 1D, red'})    
end

% Transformed data

for ord =1:10
    
    [datT Results] = PPA(dat,ord,3);
    idat = inv_PPA(datT,Results,1);
    
    figure
    
    plot3(datT(1,:),datT(2,:),datT(3,:),'b.')
    axis equal
    hold on
    plot3(datT(1,:),zeros,zeros,'r.')
    title({['Data transformed to the PPA domain, ord: ' num2str(ord)] ,'Original data, blue','Data reduced to 1D, red'})        

end


%% PPA 2D
for ord =1:10
    [datT Results] = PPA(dat,ord,3);

    %%
    idat = inv_PPA(datT,Results,2);
    
    figure
    plot3(dat(1,:),dat(2,:),dat(3,:),'b.')
    axis equal
    hold on
    plot3(idat(1,:),idat(2,:),idat(3,:),'r.')
    
    title(['PPA, 2D reduction, ord: ' num2str(ord)])    
end

%% Applying PPA to new dataset

dat2 = helix + 0.4*randn(3,length(helix(1,:)));

figure
plot3(dat(1,:),dat(2,:),dat(3,:),'b.')
hold on
axis equal
plot3(dat2(1,:),dat2(2,:),dat2(3,:),'g.')


% Learning PPA model using the first dataset

 [datT Results] = PPA(dat,10,3);

% Applying the model to the second datase

 datT2 = apply_PPA(dat2,Results);
 
figure
plot3(datT(1,:),datT(2,:),datT(3,:),'b.')
hold on
axis equal
plot3(datT2(1,:),datT2(2,:),datT2(3,:),'g.')

 
