%
% PPA
%
% [datT Results] = PPA(dat,order,DIM,TAU,initialization,tol,max_it);
%
% The function can be used in default mode to use PCA basis:
%
%    [datT Results] = PPA(dat,order,DIM);
%
% Alternatively PPA can be used optimizing the basis with Gradient Descent:
%
%    max_it = 200;
%    initialization = 1;
%    TAU = [-logspace(-6,2,10) 0 logspace(-6,2,10)];   
%    orden = 8;
%    tol = 1;
%
%    [datT Results] = PPA(dat_train,orden,DIM,TAU,initialization,tol,max_it); 
%
% INPUTS:
%
%    dat    = data #DIM x #SAMPLES
%    order  = polynomial order
%    DIM    = number of dimensions extracted
%    TAU    = vetor of tau's to be proved in the descent gradient optimization
%    initialization = 1 (PCA basis), 0 (random basis)
%    tol    = minimum error desired
%    max_it = maximum number of gradient descent iterations
%
% OUTPUTS:
%
%   datT = transformed data
%   Results = Struct with the transformation
%
% Valero Laparra, 2012 (c)
% valero.laparra@uv.es
% http://isp.uv.es
%

function [datT Results] = PPA(dat,order,DIM,TAU,initialization,tol,max_it)

%% PARAMETERS

if ~exist('order')
    order = 2;
end

if ~exist('DIM'), DIM = 1; end

if ~exist('TAU')
    TAU = 0;
end

if ~exist('initialization'),  initialization = 1; end

if ~exist('tol'), tol = 1; end

if ~exist('max_it'), max_it = 0; end


%% ALGORITHM

aux_dat = dat';

for ind_d = 1:DIM 
    %tic
    [Res err dat_trans] = PPA_1D(aux_dat,order,TAU,initialization,tol,max_it);

    Results(ind_d).Res = Res;
    Results(ind_d).err = err;
    
    datT(:,ind_d) = dat_trans(:,1);
    aux_dat = dat_trans(:,2:end);
    
    %[ind_d toc]
end

datT = datT';

