function orthonormal_basis = my_GramS(A)
%
% Gram-Schmidt Process
% 
% Gram_Schmidt_process(A) produces an orthonormal basis for the subspace of
% Eucldiean n-space spanned by the vectors {u1,u2,...}, where the matrix A 
% is formed from these vectors as its columns. That is, the subspace is the
% column space of A. The columns of the matrix that is returned are the 
% orthonormal basis vectors for the subspace. An error is returned if an
% orthonormal basis for the zero vector space is attempted to be produced.
%
% For example, if the vector space V = span{u1,u2,...}, where u1,u2,... are
% row vectors, then set A to be [u1' u2' ...].
%
% For example, if the vector space V = Col(B), where B is an m x n matrix,
% then set A to be equal to B.

[m n] = size(A);

if sum(A == zeros(m,n)) == m*n
    error('There does not exist any type of basis for the zero vector space.');
elseif n == 1
    orthonormal_basis = A(1:m,1)/norm(A(1:m,1));
else
    flag = 0;

    if ((sum(sum(A'*A-eye(size(A,2))))) == 0)
        orthonormal_basis = A;
        flag = 1;
    end

    if flag == 0;

        A = A./repmat(sqrt(sum(A.^2)),size(A,1),1);
        
        RR = A'*A;
        for dd = 2:n
            is_orth(dd) = sum(abs(RR(dd,1:dd-1)));
        end
        ii = find(is_orth<10e2*eps); % this is the "orthogonality error" assumed by eigs
        
        for i = ii(1):n
            v = A(:,i);
            M = A(:,1:i-1);
            
            v = v - sum((repmat((M'*v)',m,1).*M),2);
            A(:,i) = v/sqrt(sum(v.^2));
        end
        
        orthonormal_basis = A;
    end
end