%
% inv_PPA
%
% This funtion inverts the PPA transformation obtained with PPA.m
%
% The function can be used with full invertibility: 
%
%     idat = inv_PPA(datT,Results);
%
% Alternatively, one can also do the inversion while reducing the dimensionality (DIM)
%
% idatT = inv_PPA([datT(1:DIM,:) ; zeros(size(dat,1)-DIM,size(datT,2))] , Results(1:DIM));
%
% Valero Laparra, 2012 (c)
% valero.laparra@uv.es
% http://isp.uv.es
%

function idat = inv_PPA(datT,Results,DIM)

dim_ori = size(Results(1).Res.V,1);

if ~exist('DIM')
   DIM = size(datT,1);
end

datT = [datT(1:DIM,:) ; zeros(dim_ori-DIM,size(datT,2))];
Results = Results(1:DIM);


aux_dat = datT';

DIM = size(Results,2);

idat_trans = zeros(size(aux_dat,1),size(aux_dat,2)-DIM);

for ind_d = DIM:-1:1
    idat_trans = inv_PPA_1D(Results(ind_d).Res,[aux_dat(:,ind_d) idat_trans]);
end

idat = idat_trans';

