function [J,struct_J]=ppa_jacobian_v(x,Results)

% PPA_JACOBIAN_V computes the PPA Jacobian matrix at point x given the data
% defining the PPA transform:
%    - The series of principal directions Ep
%    - The series of principal polynomials Wp
% where p = 1,...,d-1.
% 
% This series of matrices is given to the routine in the fields of a struct 
% variable:
%             struct_ppa(p).Res.V'  (directions)
%             struct_ppa(p).Res.W'  (coeffcient matrix of the p-th polynomial) 
%
% [J,struct_J] = ppa_jacobian_v(x,struct_ppa);
%

d=length(x);

alfa = zeros(d,1);

% Traspuestas
for i=1:d-1
    struct_ppa(i).Vp = Results(i).Res.V';
    struct_ppa(i).Bp = Results(i).Res.W';
    if i<d-1
       struct_ppa(i).m = Results(i).Res.m';
    else
       struct_ppa(i).m = 0; 
    end
end

xpm1 = x - struct_ppa(1).m;

for p=1:d-1
     
    Vp=struct_ppa(p).Vp;
    Bp=struct_ppa(p).Bp;
    
    vp=Vp(1,:);
    
    alfa_p = Vp*xpm1;
    
    [dim,ord_m_1] = size(Bp); 
    
    vander_p = zeros(ord_m_1,1);
    deriv_vander_p = zeros(ord_m_1,1);
    
    vander_p(1) = 1;
    for i=2:ord_m_1     % Fucktorial
        vander_p(i) = alfa_p(1)^(i-1)/factorial(i-1);
        deriv_vander_p(i) = alfa_p(1)^(i-2)/factorial(i-2);
    end
    
    m_p = Bp*vander_p;
    
    x_p = alfa_p(2:end)-m_p;    

    alfa(p) = alfa_p(1);
    
    alfap = [alfa(1:p);x_p];
    
    if p==d-1
       alfa(p+1)=x_p;    
    end

    if p < d-1
       xpm1 = x_p - struct_ppa(p+1).m;
    end
    
    % Almaceno la construccion de la transformada del punto x
    
    struct_J(p).alfa=alfa;
    struct_J(p).alfap=alfap;
    
    % Ahora los elementos del Jacobiano!
    
    wp = Bp*deriv_vander_p;
    
    if p==1
       
       Jp =  Vp-[zeros(1,d-p+1);wp*vp];
        
    else
        
       II=eye(p-1); 
       
       Jp = [II zeros(p-1,d-p+1);zeros(d-p+1,p-1) Vp-[zeros(1,d-p+1);wp*vp]];
       
    end
    
    struct_J(p).Jp=Jp;
    
end

J=eye(d,d);

%for i=d-1:-1:1
%    A = struct_J(i).Jp;
%    J = J*A;
%end

for i=1:d-1
    A = struct_J(i).Jp;
    J = A*J;
end
