function [J,variac_residual] = ppa_jacobian_x(x,Results)

% PPA_JACOBIAN_X computes the PPA Jacobian matrix at point x given the data
% defining the PPA transform:
%    - The series of principal directions Ep
%    - The series of principal polynomials Wp
% where p = 1,...,d-1.
% 
% This series of matrices is given to the routine in the fields of a struct 
% variable:
%             struct_ppa(p).Res.V'  (directions)
%             struct_ppa(p).Res.W'  (coeffcient matrix of the p-th polynomial) 
%
% [J,variac_residual] = ppa_jacobian_x(x,struct_ppa);
%
%    J = contains the (d x d) Jacobian matrix
%    variac_residual(p).D_x_p = contains the derivative of the residual at 
%                               step p in the sequence with regard to the 
%                               input x0: i.e. (dxp/dx0)
% 


d = length(x);

for i=1:d-1
    result_ppa(i).Vp = Results(i).Res.V';
    result_ppa(i).Bp = Results(i).Res.W';
    if i<d-1
       struct_ppa(i).m = Results(i).Res.m';
    else
       struct_ppa(i).m = 0; 
    end
end

J = zeros(d);
xpm1 = x;

for p = 1:d-1
    
    Vp = result_ppa(p).Vp;
    Bp = result_ppa(p).Bp;
    
    Vp1 = Vp(1,:);
    Vp_ort = Vp(2:end,:); 
    
    alpha_p = Vp1*xpm1;        

    [dim,ord_m_1] = size(Bp); 
    vander_p = zeros(ord_m_1,1);
    deriv_vander_p = zeros(ord_m_1,1);
       
    vander_p(1) = 1;
    for i=2:ord_m_1     % Fucktorial
        vander_p(i) = alpha_p^(i-1)/factorial(i-1);
        deriv_vander_p(i) = alpha_p^(i-2)/factorial(i-2);
    end
    wp = Bp*deriv_vander_p;    
    
    xp = Vp_ort * xpm1 - Bp*vander_p;
    
    if p==1 
       
       % Primera fila del Jacobiano: derivada de alpha_1 respecto de x0  
       D_alpha_p = Vp1;
       J(p,:) = D_alpha_p;

       % Actualizacion de variables temporales
       temp(1).Vp_ort = Vp_ort;
       temp(1).wp = wp;
       temp(1).D_alpha_p = D_alpha_p;
       
    else    
       
       % Filas 2:d-1 del Jacobiano: derivada de alpha_p respecto de x0 (con p=2:d-1)     
       
       for j=p-1:-1:1 
           V_ort = temp(j).Vp_ort;
           if j==p-1
              A = V_ort;
           else
              A = A*V_ort;  
           end
       end
       
       D_alpha_p = Vp1*A;
       
       for l=1:p-2
           for j=p-1:-1:l+1 
               V_ort = temp(j).Vp_ort;
               if j==p-1
                  C = V_ort;
               else
                  C = C*V_ort;  
               end
           end
           
           wl = temp(l).wp;
           D_alpha_l = temp(l).D_alpha_p;
           
           D_alpha_p = D_alpha_p - Vp1*C*wl*D_alpha_l;
       end
       
       wpm1 = temp(p-1).wp;
       D_alpha_pm1 = temp(p-1).D_alpha_p;       
       
       D_alpha_p = D_alpha_p - Vp1*wpm1*D_alpha_pm1;
       
       J(p,:) = D_alpha_p;

       % Actualizacion de variables intermedias
       temp(p).Vp_ort = Vp_ort;
       temp(p).wp = wp;
       temp(p).D_alpha_p = D_alpha_p;
       
    end
    
    % Variacion del residuo xp respecto de x0 en el paso p de la secuencia
    % En ultimo paso de la secuencia esta variacion si es la ultima fila
    % del jacobiano global.
    % En otros pasos de la secuencia esto debe estar relacionado (de alguna manera) con el
    % Jacobiano de la transformada truncada... 
    
       for j=p:-1:1 
           V_ort = temp(j).Vp_ort;
           if j==p
              AA = V_ort;
           else
              AA = AA*V_ort;  
           end
       end
    
       D_x_p = AA;
       
       for l=1:p-1
           for j=p:-1:l+1 
               V_ort = temp(j).Vp_ort;
               if j==p
                  CC = V_ort;
               else
                  CC = CC*V_ort;  
               end
           end

           wl = temp(l).wp;
           D_alpha_l = temp(l).D_alpha_p;
           
           D_x_p = D_x_p - CC*wl*D_alpha_l;
           
       end
       
       wp = temp(p).wp;
       D_alpha_p = temp(p).D_alpha_p;       
       
       D_x_p = D_x_p - wp*D_alpha_p; 
       
       variac_residual(p).D_x_p = D_x_p;
       
       if p < d-1
          xpm1 = xp - struct_ppa(p+1).m;
       end
end

J(d,:) = D_x_p;
